@@ -1,23 +1,26 @@
 /*
  * Copyright © 2011 Intel Corporation
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
 #include "config.h"
@@ -31,6 +34,7 @@
 #include <fcntl.h>
 
 #include "compositor.h"
+#include "shared/helpers.h"
 
 WL_EXPORT void
 weston_spring_init(struct weston_spring *spring,
@@ -161,6 +165,8 @@ weston_view_animation_frame(struct weston_animation *base,
 	struct weston_view_animation *animation =
 		container_of(base,
 			     struct weston_view_animation, animation);
+	struct weston_compositor *compositor =
+		animation->view->surface->compositor;
 
 	if (base->frame_counter <= 1)
 		animation->spring.timestamp = msecs;
@@ -178,16 +184,25 @@ weston_view_animation_frame(struct weston_animation *base,
 
 	weston_view_geometry_dirty(animation->view);
 	weston_view_schedule_repaint(animation->view);
+
+	/* The view's output_mask will be zero if its position is
+	 * offscreen. Animations should always run but as they are also
+	 * run off the repaint cycle, if there's nothing to repaint
+	 * the animation stops running. Therefore if we catch this situation
+	 * and schedule a repaint on all outputs it will be avoided.
+	 */
+	if (animation->view->output_mask == 0)
+		weston_compositor_schedule_repaint(compositor);
 }
 
 static struct weston_view_animation *
-weston_view_animation_run(struct weston_view *view,
-			  float start, float stop,
-			  weston_view_animation_frame_func_t frame,
-			  weston_view_animation_frame_func_t reset,
-			  weston_view_animation_done_func_t done,
-			  void *data,
-			  void *private)
+weston_view_animation_create(struct weston_view *view,
+			     float start, float stop,
+			     weston_view_animation_frame_func_t frame,
+			     weston_view_animation_frame_func_t reset,
+			     weston_view_animation_done_func_t done,
+			     void *data,
+			     void *private)
 {
 	struct weston_view_animation *animation;
 
@@ -203,14 +218,12 @@ weston_view_animation_run(struct weston_view *view,
 	animation->start = start;
 	animation->stop = stop;
 	animation->private = private;
+
 	weston_matrix_init(&animation->transform.matrix);
 	wl_list_insert(&view->geometry.transformation_list,
 		       &animation->transform.link);
-	weston_spring_init(&animation->spring, 200.0, 0.0, 1.0);
-	animation->spring.friction = 700;
-	animation->animation.frame_counter = 0;
+
 	animation->animation.frame = weston_view_animation_frame;
-	weston_view_animation_frame(&animation->animation, NULL, 0);
 
 	animation->listener.notify = handle_animation_view_destroy;
 	wl_signal_add(&view->destroy_signal, &animation->listener);
@@ -221,6 +234,13 @@ weston_view_animation_run(struct weston_view *view,
 	return animation;
 }
 
+static void
+weston_view_animation_run(struct weston_view_animation *animation)
+{
+	animation->animation.frame_counter = 0;
+	weston_view_animation_frame(&animation->animation, NULL, 0);
+}
+
 static void
 reset_alpha(struct weston_view_animation *animation)
 {
@@ -258,9 +278,9 @@ weston_zoom_run(struct weston_view *view, float start, float stop,
 {
 	struct weston_view_animation *zoom;
 
-	zoom = weston_view_animation_run(view, start, stop,
-					 zoom_frame, reset_alpha,
-					 done, data, NULL);
+	zoom = weston_view_animation_create(view, start, stop,
+					    zoom_frame, reset_alpha,
+					    done, data, NULL);
 
 	if (zoom == NULL)
 		return NULL;
@@ -269,6 +289,8 @@ weston_zoom_run(struct weston_view *view, float start, float stop,
 	zoom->spring.friction = 1400;
 	zoom->spring.previous = start - (stop - start) * 0.03;
 
+	weston_view_animation_run(zoom);
+
 	return zoom;
 }
 
@@ -290,20 +312,21 @@ weston_fade_run(struct weston_view *view,
 {
 	struct weston_view_animation *fade;
 
-	fade = weston_view_animation_run(view, 0, end,
-					 fade_frame, reset_alpha,
-					 done, data, NULL);
+	fade = weston_view_animation_create(view, start, end,
+					    fade_frame, reset_alpha,
+					    done, data, NULL);
 
 	if (fade == NULL)
 		return NULL;
 
-	weston_spring_init(&fade->spring, k, start, end);
-
-	fade->spring.friction = 1400;
-	fade->spring.previous = -(end - start) * 0.03;
+	weston_spring_init(&fade->spring, 1000.0, start, end);
+	fade->spring.friction = 4000;
+	fade->spring.previous = start - (end - start) * 0.1;
 
 	view->alpha = start;
 
+	weston_view_animation_run(fade);
+
 	return fade;
 }
 
@@ -311,6 +334,7 @@ WL_EXPORT void
 weston_fade_update(struct weston_view_animation *fade, float target)
 {
 	fade->spring.target = target;
+	fade->stop = target;
 }
 
 static void
@@ -339,7 +363,7 @@ weston_stable_fade_run(struct weston_view *front_view, float start,
 {
 	struct weston_view_animation *fade;
 
-	fade = weston_view_animation_run(front_view, 0, 0,
+	fade = weston_view_animation_create(front_view, 0, 0,
 					    stable_fade_frame, NULL,
 					    done, data, back_view);
 
@@ -352,6 +376,8 @@ weston_stable_fade_run(struct weston_view *front_view, float start,
 	front_view->alpha = start;
 	back_view->alpha = end;
 
+	weston_view_animation_run(fade);
+
 	return fade;
 }
 
@@ -373,16 +399,18 @@ weston_slide_run(struct weston_view *view, float start, float stop,
 {
 	struct weston_view_animation *animation;
 
-	animation = weston_view_animation_run(view, start, stop,
+	animation = weston_view_animation_create(view, start, stop,
 					      slide_frame, NULL, done,
 					      data, NULL);
 	if (!animation)
 		return NULL;
 
+	weston_spring_init(&animation->spring, 400.0, 0.0, 1.0);
 	animation->spring.friction = 600;
-	animation->spring.k = 400;
 	animation->spring.clip = WESTON_SPRING_BOUNCE;
 
+	weston_view_animation_run(animation);
+
 	return animation;
 }
 
@@ -440,14 +468,16 @@ weston_move_scale_run(struct weston_view *view, int dx, int dy,
 	move->reverse = reverse;
 	move->done = done;
 
-	animation = weston_view_animation_run(view, start, end, move_frame,
-	                                      NULL, move_done, data, move);
+	animation = weston_view_animation_create(view, start, end, move_frame,
+						 NULL, move_done, data, move);
 
 	if (animation == NULL)
 		return NULL;
 
-	animation->spring.k = 400;
+	weston_spring_init(&animation->spring, 400.0, 0.0, 1.0);
 	animation->spring.friction = 1150;
 
+	weston_view_animation_run(animation);
+
 	return animation;
 }
