@@ -1,47 +1,55 @@
 /*
  * Copyright © 2012 Intel Corporation
  *
- * Permission to use, copy, modify, distribute, and sell this software and its
- * documentation for any purpose is hereby granted without fee, provided that
- * the above copyright notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting documentation, and
- * that the name of the copyright holders not be used in advertising or
- * publicity pertaining to distribution of the software without specific,
- * written prior permission.  The copyright holders make no representations
- * about the suitability of this software for any purpose.  It is provided "as
- * is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject to
+ * the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
- * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
- * EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL, INDIRECT OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
- * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
- * OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the
+ * next paragraph) shall be included in all copies or substantial
+ * portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+ * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
+ * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
+ * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <errno.h>
 #include <sys/mman.h>
 
-#include "../shared/os-compatibility.h"
+#include "shared/os-compatibility.h"
 #include "weston-test-client-helper.h"
 
-static inline void *
-xzalloc(size_t size)
+#define max(a, b) (((a) > (b)) ? (a) : (b))
+#define min(a, b) (((a) > (b)) ? (b) : (a))
+#define clip(x, a, b)  min(max(x, a), b)
+
+void *
+fail_on_null(void *p)
 {
-	void *p;
-
-	p = calloc(1, size);
-	assert(p);
-
+	if (p == NULL) {
+		fprintf(stderr, "out of memory\n");
+		exit(EXIT_FAILURE);
+	}
 	return p;
 }
 
+
 int
 surface_contains(struct surface *surface, int x, int y)
 {
@@ -79,12 +87,15 @@ frame_callback_set(struct wl_surface *surface, int *done)
 	return callback;
 }
 
-void
-frame_callback_wait(struct client *client, int *done)
+int
+frame_callback_wait_nofail(struct client *client, int *done)
 {
 	while (!*done) {
-		assert(wl_display_dispatch(client->wl_display) >= 0);
+		if (wl_display_dispatch(client->wl_display) < 0)
+			return 0;
 	}
+
+	return 1;
 }
 
 void
@@ -95,9 +106,9 @@ move_client(struct client *client, int x, int y)
 
 	client->surface->x = x;
 	client->surface->y = y;
-	wl_test_move_surface(client->test->wl_test, surface->wl_surface,
+	weston_test_move_surface(client->test->weston_test, surface->wl_surface,
 			     surface->x, surface->y);
-	/* The attach here is necessary because commit() will call congfigure
+	/* The attach here is necessary because commit() will call configure
 	 * only on surfaces newly attached, and the one that sets the surface
 	 * position is the configure. */
 	wl_surface_attach(surface->wl_surface, surface->wl_buffer, 0, 0);
@@ -116,7 +127,7 @@ get_n_egl_buffers(struct client *client)
 {
 	client->test->n_egl_buffers = -1;
 
-	wl_test_get_n_egl_buffers(client->test->wl_test);
+	weston_test_get_n_egl_buffers(client->test->weston_test);
 	wl_display_roundtrip(client->wl_display);
 
 	return client->test->n_egl_buffers;
@@ -256,12 +267,91 @@ keyboard_handle_modifiers(void *data, struct wl_keyboard *wl_keyboard,
 		mods_depressed, mods_latched, mods_locked, group);
 }
 
+static void
+keyboard_handle_repeat_info(void *data, struct wl_keyboard *wl_keyboard,
+			    int32_t rate, int32_t delay)
+{
+	struct keyboard *keyboard = data;
+
+	keyboard->repeat_info.rate = rate;
+	keyboard->repeat_info.delay = delay;
+
+	fprintf(stderr, "test-client: got keyboard repeat_info %d %d\n",
+		rate, delay);
+}
+
 static const struct wl_keyboard_listener keyboard_listener = {
 	keyboard_handle_keymap,
 	keyboard_handle_enter,
 	keyboard_handle_leave,
 	keyboard_handle_key,
 	keyboard_handle_modifiers,
+	keyboard_handle_repeat_info,
+};
+
+static void
+touch_handle_down(void *data, struct wl_touch *wl_touch,
+		  uint32_t serial, uint32_t time, struct wl_surface *surface,
+		  int32_t id, wl_fixed_t x_w, wl_fixed_t y_w)
+{
+	struct touch *touch = data;
+
+	touch->down_x = wl_fixed_to_int(x_w);
+	touch->down_y = wl_fixed_to_int(y_w);
+	touch->id = id;
+
+	fprintf(stderr, "test-client: got touch down %d %d, surf: %p, id: %d\n",
+		touch->down_x, touch->down_y, surface, id);
+}
+
+static void
+touch_handle_up(void *data, struct wl_touch *wl_touch,
+		uint32_t serial, uint32_t time, int32_t id)
+{
+	struct touch *touch = data;
+	touch->up_id = id;
+
+	fprintf(stderr, "test-client: got touch up, id: %d\n", id);
+}
+
+static void
+touch_handle_motion(void *data, struct wl_touch *wl_touch,
+		    uint32_t time, int32_t id, wl_fixed_t x_w, wl_fixed_t y_w)
+{
+	struct touch *touch = data;
+	touch->x = wl_fixed_to_int(x_w);
+	touch->y = wl_fixed_to_int(y_w);
+
+	fprintf(stderr, "test-client: got touch motion, %d %d, id: %d\n",
+		touch->x, touch->y, id);
+}
+
+static void
+touch_handle_frame(void *data, struct wl_touch *wl_touch)
+{
+	struct touch *touch = data;
+
+	++touch->frame_no;
+
+	fprintf(stderr, "test-client: got touch frame (%d)\n", touch->frame_no);
+}
+
+static void
+touch_handle_cancel(void *data, struct wl_touch *wl_touch)
+{
+	struct touch *touch = data;
+
+	++touch->cancel_no;
+
+	fprintf(stderr, "test-client: got touch cancel (%d)\n", touch->cancel_no);
+}
+
+static const struct wl_touch_listener touch_listener = {
+	touch_handle_down,
+	touch_handle_up,
+	touch_handle_motion,
+	touch_handle_frame,
+	touch_handle_cancel,
 };
 
 static void
@@ -340,7 +430,7 @@ struct wl_shm_listener shm_listener = {
 };
 
 static void
-test_handle_pointer_position(void *data, struct wl_test *wl_test,
+test_handle_pointer_position(void *data, struct weston_test *weston_test,
 			     wl_fixed_t x, wl_fixed_t y)
 {
 	struct test *test = data;
@@ -352,25 +442,37 @@ test_handle_pointer_position(void *data, struct wl_test *wl_test,
 }
 
 static void
-test_handle_n_egl_buffers(void *data, struct wl_test *wl_test, uint32_t n)
+test_handle_n_egl_buffers(void *data, struct weston_test *weston_test, uint32_t n)
 {
 	struct test *test = data;
 
 	test->n_egl_buffers = n;
 }
 
-static const struct wl_test_listener test_listener = {
+static void
+test_handle_capture_screenshot_done(void *data, struct weston_test *weston_test)
+{
+	struct test *test = data;
+
+	printf("Screenshot has been captured\n");
+	test->buffer_copy_done = 1;
+}
+
+static const struct weston_test_listener test_listener = {
 	test_handle_pointer_position,
 	test_handle_n_egl_buffers,
+	test_handle_capture_screenshot_done,
 };
 
 static void
-seat_handle_capabilities(void *data, struct wl_seat *seat,
-			 enum wl_seat_capability caps)
+input_update_devices(struct input *input)
 {
-	struct input *input = data;
 	struct pointer *pointer;
 	struct keyboard *keyboard;
+	struct touch *touch;
+
+	struct wl_seat *seat = input->wl_seat;
+	enum wl_seat_capability caps = input->caps;
 
 	if ((caps & WL_SEAT_CAPABILITY_POINTER) && !input->pointer) {
 		pointer = xzalloc(sizeof *pointer);
@@ -397,10 +499,54 @@ seat_handle_capabilities(void *data, struct wl_seat *seat,
 		free(input->keyboard);
 		input->keyboard = NULL;
 	}
+
+	if ((caps & WL_SEAT_CAPABILITY_TOUCH) && !input->touch) {
+		touch = xzalloc(sizeof *touch);
+		touch->wl_touch = wl_seat_get_touch(seat);
+		wl_touch_set_user_data(touch->wl_touch, touch);
+		wl_touch_add_listener(touch->wl_touch, &touch_listener,
+					 touch);
+		input->touch = touch;
+	} else if (!(caps & WL_SEAT_CAPABILITY_TOUCH) && input->touch) {
+		wl_touch_destroy(input->touch->wl_touch);
+		free(input->touch);
+		input->touch = NULL;
+	}
+}
+
+static void
+seat_handle_capabilities(void *data, struct wl_seat *seat,
+			 enum wl_seat_capability caps)
+{
+	struct input *input = data;
+
+	input->caps = caps;
+
+	/* we will create/update the devices only with the right (test) seat.
+	 * If we haven't discovered which seat is the test seat, just
+	 * store capabilities and bail out */
+	if (input->seat_name && strcmp(input->seat_name, "test-seat") == 0)
+		input_update_devices(input);
+
+	fprintf(stderr, "test-client: got seat %p capabilities: %x\n",
+		input, caps);
+}
+
+static void
+seat_handle_name(void *data, struct wl_seat *seat, const char *name)
+{
+	struct input *input = data;
+
+	input->seat_name = strdup(name);
+	assert(input->seat_name && "No memory");
+
+	fprintf(stderr, "test-client: got seat %p name: \'%s\'\n",
+		input, name);
 }
 
 static const struct wl_seat_listener seat_listener = {
 	seat_handle_capabilities,
+	seat_handle_name,
 };
 
 static void
@@ -436,9 +582,30 @@ output_handle_mode(void *data,
 	}
 }
 
+static void
+output_handle_scale(void *data,
+		    struct wl_output *wl_output,
+		    int scale)
+{
+	struct output *output = data;
+
+	output->scale = scale;
+}
+
+static void
+output_handle_done(void *data,
+		   struct wl_output *wl_output)
+{
+	struct output *output = data;
+
+	output->initialized = 1;
+}
+
 static const struct wl_output_listener output_listener = {
 	output_handle_geometry,
-	output_handle_mode
+	output_handle_mode,
+	output_handle_done,
+	output_handle_scale,
 };
 
 static void
@@ -446,10 +613,10 @@ handle_global(void *data, struct wl_registry *registry,
 	      uint32_t id, const char *interface, uint32_t version)
 {
 	struct client *client = data;
-	struct input *input;
 	struct output *output;
 	struct test *test;
 	struct global *global;
+	struct input *input;
 
 	global = xzalloc(sizeof *global);
 	global->name = id;
@@ -461,34 +628,36 @@ handle_global(void *data, struct wl_registry *registry,
 	if (strcmp(interface, "wl_compositor") == 0) {
 		client->wl_compositor =
 			wl_registry_bind(registry, id,
-					 &wl_compositor_interface, 1);
+					 &wl_compositor_interface, version);
 	} else if (strcmp(interface, "wl_seat") == 0) {
 		input = xzalloc(sizeof *input);
 		input->wl_seat =
 			wl_registry_bind(registry, id,
-					 &wl_seat_interface, 1);
+					 &wl_seat_interface, version);
 		wl_seat_add_listener(input->wl_seat, &seat_listener, input);
-		client->input = input;
+		wl_list_insert(&client->inputs, &input->link);
 	} else if (strcmp(interface, "wl_shm") == 0) {
 		client->wl_shm =
 			wl_registry_bind(registry, id,
-					 &wl_shm_interface, 1);
+					 &wl_shm_interface, version);
 		wl_shm_add_listener(client->wl_shm, &shm_listener, client);
 	} else if (strcmp(interface, "wl_output") == 0) {
 		output = xzalloc(sizeof *output);
 		output->wl_output =
 			wl_registry_bind(registry, id,
-					 &wl_output_interface, 1);
+					 &wl_output_interface, version);
 		wl_output_add_listener(output->wl_output,
 				       &output_listener, output);
 		client->output = output;
-	} else if (strcmp(interface, "wl_test") == 0) {
+	} else if (strcmp(interface, "weston_test") == 0) {
 		test = xzalloc(sizeof *test);
-		test->wl_test =
+		test->weston_test =
 			wl_registry_bind(registry, id,
-					 &wl_test_interface, 1);
-		wl_test_add_listener(test->wl_test, &test_listener, test);
+					 &weston_test_interface, version);
+		weston_test_add_listener(test->weston_test, &test_listener, test);
 		client->test = test;
+	} else if (strcmp(interface, "wl_drm") == 0) {
+		client->has_wl_drm = true;
 	}
 }
 
@@ -505,9 +674,58 @@ skip(const char *fmt, ...)
 	vfprintf(stderr, fmt, argp);
 	va_end(argp);
 
-	/* automake tests uses exit code 77, but we don't have a good
-	 * way to make weston exit with that from here. */
-	exit(0);
+	/* automake tests uses exit code 77. weston-test-runner will see
+	 * this and use it, and then weston-test's sigchld handler (in the
+	 * weston process) will use that as an exit status, which is what
+	 * automake will see in the end. */
+	exit(77);
+}
+
+void
+expect_protocol_error(struct client *client,
+		      const struct wl_interface *intf,
+		      uint32_t code)
+{
+	int err;
+	uint32_t errcode, failed = 0;
+	const struct wl_interface *interface;
+	unsigned int id;
+
+	/* if the error has not come yet, make it happen */
+	wl_display_roundtrip(client->wl_display);
+
+	err = wl_display_get_error(client->wl_display);
+
+	assert(err && "Expected protocol error but nothing came");
+	assert(err == EPROTO && "Expected protocol error but got local error");
+
+	errcode = wl_display_get_protocol_error(client->wl_display,
+						&interface, &id);
+
+	/* check error */
+	if (errcode != code) {
+		fprintf(stderr, "Should get error code %d but got %d\n",
+			code, errcode);
+		failed = 1;
+	}
+
+	/* this should be definitely set */
+	assert(interface);
+
+	if (strcmp(intf->name, interface->name) != 0) {
+		fprintf(stderr, "Should get interface '%s' but got '%s'\n",
+			intf->name, interface->name);
+		failed = 1;
+	}
+
+	if (failed) {
+		fprintf(stderr, "Expected other protocol error\n");
+		abort();
+	}
+
+	/* all OK */
+	fprintf(stderr, "Got expected protocol error on '%s' (object id: %d) "
+			"with code %d\n", interface->name, id, errcode);
 }
 
 static void
@@ -517,11 +735,38 @@ log_handler(const char *fmt, va_list args)
 	vfprintf(stderr, fmt, args);
 }
 
+static void
+input_destroy(struct input *inp)
+{
+	wl_list_remove(&inp->link);
+	wl_seat_destroy(inp->wl_seat);
+	free(inp);
+}
+
+/* find the test-seat and set it in client.
+ * Destroy other inputs */
+static void
+client_set_input(struct client *cl)
+{
+	struct input *inp, *inptmp;
+	wl_list_for_each_safe(inp, inptmp, &cl->inputs, link) {
+		assert(inp->seat_name && "BUG: input with no name");
+		if (strcmp(inp->seat_name, "test-seat") == 0) {
+			cl->input = inp;
+			input_update_devices(inp);
+		} else {
+			input_destroy(inp);
+		}
+	}
+
+	/* we keep only one input */
+	assert(wl_list_length(&cl->inputs) == 1);
+}
+
 struct client *
-client_create(int x, int y, int width, int height)
+create_client(void)
 {
 	struct client *client;
-	struct surface *surface;
 
 	wl_log_set_handler_client(log_handler);
 
@@ -530,24 +775,48 @@ client_create(int x, int y, int width, int height)
 	client->wl_display = wl_display_connect(NULL);
 	assert(client->wl_display);
 	wl_list_init(&client->global_list);
+	wl_list_init(&client->inputs);
 
 	/* setup registry so we can bind to interfaces */
 	client->wl_registry = wl_display_get_registry(client->wl_display);
-	wl_registry_add_listener(client->wl_registry, &registry_listener, client);
+	wl_registry_add_listener(client->wl_registry, &registry_listener,
+				 client);
 
-	/* trigger global listener */
-	wl_display_dispatch(client->wl_display);
-	wl_display_roundtrip(client->wl_display);
+	/* this roundtrip makes sure we have all globals and we bound to them */
+	client_roundtrip(client);
+	/* this roundtrip makes sure we got all wl_shm.format and wl_seat.*
+	 * events */
+	client_roundtrip(client);
+
+	/* find the right input for us */
+	client_set_input(client);
 
 	/* must have WL_SHM_FORMAT_ARGB32 */
 	assert(client->has_argb);
 
-	/* must have wl_test interface */
+	/* must have weston_test interface */
 	assert(client->test);
 
 	/* must have an output */
 	assert(client->output);
 
+	/* the output must be initialized */
+	assert(client->output->initialized == 1);
+
+	/* must have seat set */
+	assert(client->input);
+
+	return client;
+}
+
+struct client *
+create_client_and_test_surface(int x, int y, int width, int height)
+{
+	struct client *client;
+	struct surface *surface;
+
+	client = create_client();
+
 	/* initialize the client surface */
 	surface = xzalloc(sizeof *surface);
 	surface->wl_surface =
@@ -571,3 +840,142 @@ client_create(int x, int y, int width, int height)
 
 	return client;
 }
+
+static const char*
+output_path(void)
+{
+	char *path = getenv("WESTON_TEST_OUTPUT_PATH");
+
+	if (!path)
+		return ".";
+	return path;
+	}
+
+char*
+screenshot_output_filename(const char *basename, uint32_t seq)
+{
+	char *filename;
+
+	if (asprintf(&filename, "%s/%s-%02d.png",
+				 output_path(), basename, seq) < 0)
+		return NULL;
+	return filename;
+}
+
+static const char*
+reference_path(void)
+{
+	char *path = getenv("WESTON_TEST_REFERENCE_PATH");
+
+	if (!path)
+		return "./tests/reference";
+	return path;
+}
+
+char*
+screenshot_reference_filename(const char *basename, uint32_t seq)
+{
+	char *filename;
+
+	if (asprintf(&filename, "%s/%s-%02d.png",
+				 reference_path(), basename, seq) < 0)
+		return NULL;
+	return filename;
+}
+
+/**
+ * check_surfaces_geometry() - verifies two surfaces are same size
+ *
+ * @returns true if surfaces have the same width and height, or false
+ * if not, or if there is no actual data.
+ */
+bool
+check_surfaces_geometry(const struct surface *a, const struct surface *b)
+{
+	if (a == NULL || b == NULL) {
+		printf("Undefined surfaces\n");
+		return false;
+	}
+	else if (a->data == NULL || b->data == NULL) {
+		printf("Undefined data\n");
+		return false;
+	}
+	else if (a->width != b->width || a->height != b->height) {
+		printf("Mismatched dimensions:  %d,%d != %d,%d\n",
+		       a->width, a->height, b->width, b->height);
+		return false;
+	}
+	return true;
+}
+
+/**
+ * check_surfaces_equal() - tests if two surfaces are pixel-identical
+ *
+ * Returns true if surface buffers have all the same byte values,
+ * false if the surfaces don't match or can't be compared due to
+ * different dimensions.
+ */
+bool
+check_surfaces_equal(const struct surface *a, const struct surface *b)
+{
+	int bpp = 4;  /* Assumes ARGB */
+
+	if (!check_surfaces_geometry(a, b))
+		return false;
+
+	return (memcmp(a->data, b->data, bpp * a->width * a->height) == 0);
+}
+
+/**
+ * check_surfaces_match_in_clip() - tests if a given region within two
+ * surfaces are pixel-identical.
+ *
+ * Returns true if the two surfaces have the same byte values within the
+ * given clipping region, or false if they don't match or the surfaces
+ * can't be compared.
+ */
+bool
+check_surfaces_match_in_clip(const struct surface *a, const struct surface *b, const struct rectangle *clip_rect)
+{
+	int i, j;
+	int x0, y0, x1, y1;
+	void *p, *q;
+	int bpp = 4;  /* Assumes ARGB */
+
+	if (!check_surfaces_geometry(a, b) || clip_rect == NULL)
+		return false;
+
+	if (clip_rect->x > a->width || clip_rect->y > a->height) {
+		printf("Clip outside image boundaries\n");
+		return true;
+	}
+
+	x0 = max(0, clip_rect->x);
+	y0 = max(0, clip_rect->y);
+	x1 = min(a->width,  clip_rect->x + clip_rect->width);
+	y1 = min(a->height, clip_rect->y + clip_rect->height);
+
+	if (x0 == x1 || y0 == y1) {
+		printf("Degenerate comparison\n");
+		return true;
+	}
+
+	printf("Bytewise comparison inside clip\n");
+	for (i=y0; i<y1; i++) {
+		p = a->data + i * a->width * bpp + x0 * bpp;
+		q = b->data + i * b->width * bpp + x0 * bpp;
+		if (memcmp(p, q, (x1-x0)*bpp) != 0) {
+			/* Dump the bad row */
+			printf("Mismatched image on row %d\n", i);
+			for (j=0; j<(x1-x0)*bpp; j++) {
+				char a_char = *((char*)(p+j*bpp));
+				char b_char = *((char*)(q+j*bpp));
+				printf("%d,%d: %8x %8x %s\n", i, j, a_char, b_char,
+				       (a_char != b_char)? " <---": "");
+			}
+			return false;
+		}
+	}
+
+	return true;
+}
